@@ -11,6 +11,7 @@ import fnmatch
 from typing import List, Optional, Union, Dict, Set
 
 import numpy as np
+import pandas as pd
 
 import ROOT
 
@@ -19,6 +20,7 @@ from quickstats import semistaticmethod
 from quickstats.components import AbstractObject
 from quickstats.components.numerics import is_integer, pretty_value
 from quickstats.utils.root_utils import load_macro
+from quickstats.utils.roofit_utils import get_variable_attributes, variable_collection_to_dataframe
 
 class ExtendedModel(AbstractObject):
     
@@ -603,8 +605,10 @@ class ExtendedModel(AbstractObject):
         constrained_nuis, _, _ = self.pair_constraints()
         return constrained_nuis
     
-    def get_unconstrained_nuisance_parameters(self):
-        constrained_nuis = self.get_constrained_nuisance_parameters()
+    def get_unconstrained_nuisance_parameters(self, constrained_nuis=None):
+        # do not re-isolate constrained nuisance parameters
+        if constrained_nuis is None:
+            constrained_nuis = self.get_constrained_nuisance_parameters()
         unconstrained_nuis = list(set(self.nuisance_parameters) - set(constrained_nuis))
         return unconstrained_nuis
     
@@ -1086,13 +1090,6 @@ class ExtendedModel(AbstractObject):
     def save(self, fname:str, recreate:bool=True):
         self.workspace.writeToFile(fname, recreate)
     
-    @staticmethod
-    def to_dataframe(args):
-        import pandas as pd
-        data = [{'Name':i.GetName(), 'Value':i.getVal(), "Constant":i.isConstant(), "Min":i.getMin(), "Max":i.getMax()} for i in args]
-        df = pd.DataFrame(data)
-        return df
-    
     @semistaticmethod
     def load_ws(self, fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None):
         if not os.path.exists(fname):
@@ -1328,7 +1325,29 @@ class ExtendedModel(AbstractObject):
         ylabel = "Events"
         return plot_distributions(collected_distributions[category], xlabel=xlabel, ylabel=ylabel)
     
-    def print_summary(self, items:Optional[List]=None, suppress_print:bool=False, save_as:Optional[str]=None):
+    def as_dataframe(self, variable_type:str):
+        if variable_type == "pois":
+            vcollection = self.pois
+        elif variable_type == "nuisance_parameters":
+            vcollection = self.nuisance_parameters
+        elif variable_type == "global_observables":
+            vcollection = self.global_observables
+        elif variable_type == "constrained_nuisance_parameters":
+            vcollection = self.get_constrained_nuisance_parameters()
+        elif variable_type == "unconstrained_nuisance_parameters":
+            vcollection = self.get_unconstrained_nuisance_parameters()
+        elif variable_type == "constraints":
+            data = self.pair_constraints(to_str=True)
+            df = pd.DataFrame(data, columns =['pdf', 'nuisance parameters', 'global observables'])
+            return df
+        else:
+            raise ValueError("unknown variable type")
+        df = variable_collection_to_dataframe(vcollection)
+        return df    
+    
+    def print_summary(self, items:Optional[List]=None, suppress_print:bool=False,
+                      include_patterns:Optional[List]=None, exclude_patterns:Optional[List]=None,
+                      save_as:Optional[str]=None):
         if items is None:
             items = ['workspace', 'dataset', 'category', 'poi', 'detailed_nuisance_parameter']
         summary_str = ""
@@ -1339,14 +1358,14 @@ class ExtendedModel(AbstractObject):
         # datasets
         if 'dataset' in items:
             datasets = self.workspace.allData()
-            summary_str += f"Datasets:\n"
+            summary_str += f"Datasets ({len(datasets)}):\n"
             summary_str += "\t" + ", ".join([d.GetName() for d in datasets]) + "\n"
         # categories
         if 'category' in items:
             if isinstance(self.pdf, ROOT.RooSimultaneous):
                 channel_cat = self.pdf.indexCat()
                 n_cat = len(channel_cat)
-                summary_str += f"Categories:\n"
+                summary_str += f"Categories ({n_cat}):\n"
                 for i in range(n_cat):
                     channel_cat.setIndex(i)
                     label = channel_cat.getLabel()
@@ -1360,7 +1379,7 @@ class ExtendedModel(AbstractObject):
             param_sets.append(pois)
         if 'detailed_nuisance_parameter' in items:
             constrained_nps = self.get_constrained_nuisance_parameters()
-            unconstrained_nps = self.get_unconstrained_nuisance_parameters()            
+            unconstrained_nps = self.get_unconstrained_nuisance_parameters(constrained_nps)            
             param_strs += ['Constrained NPs', 'Unconstrained NPs']
             param_sets += [constrained_nps, unconstrained_nps]
         elif 'nuisance_parameter' in items:
@@ -1370,7 +1389,11 @@ class ExtendedModel(AbstractObject):
             param_strs += ['Global Observables']
             param_sets += [self.global_observables]
         for pstr, pset in zip(param_strs, param_sets):
-            summary_str += f"{pstr}:\n"
+            summary_str += f"{pstr} ({len(pset)}):\n"
+            if include_patterns is not None:
+                pset = [p for pattern in include_patterns for p in pset if fnmatch.fnmatch(p.GetName(), pattern)]
+            if exclude_patterns is not None:
+                pset = [p for pattern in exclude_patterns for p in pset if not fnmatch.fnmatch(p.GetName(), pattern)]
             for p in pset:
                 name   = p.GetName()
                 value  = p.getVal()
