@@ -18,7 +18,7 @@ def load_cache(fname:str):
     else:
         return None
 
-def evaluate_nll(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax:float=10., unconditional=False,
+def evaluate_nll_old(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax:float=10., unconditional=False,
                  workspace:Optional[str]=None, model_config:Optional[str]=None, data:str='combData', snapshot:str=None, 
                  profile:str="", fix:str="", hesse:bool=False, minos:bool=True, constrain:bool=True, 
                  minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, binned:bool=True,
@@ -116,7 +116,7 @@ def evaluate_nll(input_file:str, poi_val:float, poi:str="", vmin:float=10., vmax
         return results
     return nll
 
-def scan_nll(input_file:str, poi_min:float, poi_max:float, poi_step:float, poi:str="", cache:bool=True,
+def scan_nll_old(input_file:str, poi_min:float, poi_max:float, poi_step:float, poi:Optional[str]=None, cache:bool=True,
              outname:str="{poi}", outdir:str='output', vmin:float=10., vmax:float=10., 
              workspace:Optional[str]=None, model_config:Optional[str]=None, data:str='combData', snapshot:str=None, 
              profile:str="", fix:str="", hesse:bool=False, minos:bool=True, constrain:bool=True,
@@ -128,7 +128,7 @@ def scan_nll(input_file:str, poi_min:float, poi_max:float, poi_step:float, poi:s
     points        = np.arange(poi_min, poi_max+poi_step, poi_step)
     unconditional = np.concatenate(([True], np.full((len(points)), False)))
     points        = np.concatenate(([0.], points))
-    if not poi:
+    if poi is None:
         poi = ExtendedModel.get_poi_names(input_file, workspace, model_config)[0]
         print('INFO: POI not given, default as "{}".'.format(poi))
     # try to get the number of significant digits of poi_step
@@ -159,8 +159,7 @@ def scan_nll(input_file:str, poi_min:float, poi_max:float, poi_step:float, poi:s
     with open(final_outname, 'w') as outfile:
         json.dump(data, outfile, indent=3)
     end_time = time.time()
-    print('INFO: All jobs have finished. Total Time taken: {:.3f} s'.format(end_time-start_time))
-    
+    print('INFO: All jobs have finished. Total Time taken: {:.3f} s'.format(end_time-start_time))  
 
 from quickstats.components import AnalysisObject
 from quickstats.utils.common_utils import parse_config
@@ -175,7 +174,7 @@ class Likelihood(AnalysisObject):
     def minNll(self):
         if not self.minimizer.fit_result:
             return None
-        return self.minimizer.fit_result.minNll()
+        return self.minimizer.nll.getVal()
     
     @property
     def status(self):
@@ -204,7 +203,7 @@ class Likelihood(AnalysisObject):
             self.poi.setConstant(1)
         self.minimizer.minimize()
         
-        return self.minNll
+        return self.minimizer.nll.getVal()
         
     def evaluate(self, poi_val:float, vmin:float=10., vmax:float=10., 
                  unconditional=False, hesse:bool=False, minos:bool=False, 
@@ -233,5 +232,123 @@ class Likelihood(AnalysisObject):
 
         # perform the fit
         self.minimizer.minimize(**minimizer_options)
+        
+        return self.minimizer.nll.getVal()
+    
+def evaluate_nll(filename:str, poi_val:Optional[float]=None, poi_name:str=None,
+                 ws_name:Optional[str]=None, mc_name:Optional[str]=None, data_name:str='combData', 
+                 snapshot_name:str=None, profile_param:str="", fix_param:str="", 
+                 constrain_nuis:bool=True, minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', 
+                 num_cpu:int=1, binned_likelihood:bool=True, eps:float=1.0, strategy:int=1, 
+                 optimize:int=2, offset:bool=False, fix_cache:bool=True, fix_multi:bool=True, 
+                 max_calls:int=-1, max_iters:int=-1, print_level:int=1, verbosity:int="INFO",  
+                 outname:str=None, cache:bool=False, detailed_output:bool=False):
+    if cache:
+        cached_result = load_cache(outname)
+        if cached_result is not None:
+            poi_name, nll = cached_result['poi'], cached_result['nll']
+            print('INFO: Found NLL cache from "{}"'.format(outname))
+            if poi_val is None:
+                print('INFO: Cached unconditional NLL for POI "{}": {}'.format(poi_name, nll))
+            else:
+                print('INFO: Cached NLL for POI "{}" at {:.2f}: {}'.format(poi_name, poi_val, nll))
+            return nll
+    start = time.time()    
+    config = {
+        'ws_name' : ws_name,
+        'mc_name' : mc_name,
+        'snapshot_name': snapshot_name,
+        'profile_param': profile_param,
+        'fix_param': fix_param,
+        'constrain_nuis': constrain_nuis,
+        'minimizer_type': minimizer_type,
+        'minimizer_algo': minimizer_algo,
+        'num_cpu': num_cpu,
+        'binned_likelihood': binned_likelihood,
+        'eps': eps,
+        'strategy': strategy,
+        'optimize': optimize,
+        'offset': offset,
+        'fix_cache': fix_cache,
+        'fix_multi': fix_multi,
+        'max_calls': max_calls,
+        'max_iters': max_iters,
+        'print_level': print_level,
+        'verbosity': verbosity,
+    }    
+    likelihood = Likelihood(filename=filename, poi_name=poi_name, data_name=data_name, config=config)
+    poi_name = likelihood.poi.GetName()
+    nll = likelihood._evaluate(poi_val)
+    end = time.time()
+    if poi_val is None:
+        print('INFO: Unconditional NLL for POI "{}": {}'.format(poi_name, nll))
+    else:
+        print('INFO: NLL for POI "{}" at {:.2f}: {}'.format(poi_name, poi_val, nll))
+    print('INFO: Time taken to evaluate NLL: {:.3f} s'.format(end-start))
 
-        return self.minNll
\ No newline at end of file
+    results = {
+        'nll': nll,
+        'poi': poi_name,
+        'constrain_nuis': int(constrain_nuis),
+        'poi_value': poi_val
+    }
+    if poi_val is None:
+        poi_val = likelihood.poi.getVal()
+        results['poi_bestfit'] = poi_val
+    results['time'] = end-start
+
+    # save results
+    if outname is not None:
+        with open(outname, 'w') as outfile:
+            json.dump(results, outfile)
+        print('INFO: Saved NLL result to {}'.format(outname))
+
+    if detailed_output:
+        return results
+    return nll
+
+def scan_nll(filename:str, poi_min:float, poi_max:float, poi_step:float, poi_name:Optional[str]=None, 
+             cache:bool=True, outname:str="{poi_name}", outdir:str='output', ws_name:Optional[str]=None, 
+             mc_name:Optional[str]=None, data_name:str='combData', snapshot_name:str=None, 
+             profile_param:str="", fix_param:str="", constrain_nuis:bool=True,
+             minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', num_cpu:int=1, binned_likelihood:bool=True,
+             eps:float=1.0, strategy:int=0, fix_cache:bool=True, fix_multi:bool=True, verbosity:int="INFO", 
+             print_level:int=1, max_calls:int=-1, max_iters:int=-1, optimize:int=2, offset:bool=True, parallel:int=-1):
+    start_time = time.time()
+    points        = np.arange(poi_min, poi_max+poi_step, poi_step)
+    points        = np.concatenate(([None], points))
+    # try to get the number of significant digits of poi_step
+    sd = approx_n_digit(poi_step)
+    if poi_name is None:
+        poi_name = ExtendedModel.get_poi_names(filename, ws_name, mc_name)[0]
+        print('INFO: POI not given, default as "{}".'.format(poi_name))    
+    # save cache file as outdir/cache/outname_{str_encoded_poi_value}.json
+    outnames = ["{}_{}.json".format(outname.format(poi_name=poi_name), "uncond" if point is None \
+                else str_encode_value(point, sd)) for point in points]
+    cachedir = os.path.join(outdir, "cache")
+    outnames = [os.path.join(cachedir, name) for name in outnames]
+    if not os.path.exists(outdir):
+        os.makedirs(outdir)
+    if not os.path.exists(cachedir):
+        os.makedirs(cachedir)
+
+    arguments = (repeat(filename), points, repeat(poi_name), repeat(ws_name),repeat(mc_name), 
+                 repeat(data_name), repeat(snapshot_name), repeat(profile_param), repeat(fix_param), 
+                 repeat(constrain_nuis), repeat(minimizer_type), repeat(minimizer_algo), repeat(num_cpu),
+                 repeat(binned_likelihood), repeat(eps), repeat(strategy), repeat(optimize), repeat(offset),
+                 repeat(fix_cache), repeat(fix_multi), repeat(max_calls), repeat(max_iters), 
+                 repeat(print_level), repeat(verbosity), outnames, repeat(cache), repeat(False))
+    
+    results = common_utils.execute_multi_tasks(evaluate_nll, *arguments, parallel=parallel)
+
+    uncond_nll = results[0]
+    data = {'mu': [], 'nll':[], 'qmu':[]}
+    for mu, nll in zip(points, results):
+        data['mu'].append(mu)
+        data['nll'].append(nll)
+        data['qmu'].append(2*(nll-uncond_nll))
+    final_outname = os.path.join(outdir, outname.format(poi_name=poi_name) + '.json')                  
+    with open(final_outname, 'w') as outfile:
+        json.dump(data, outfile, indent=3)
+    end_time = time.time()
+    print('INFO: All jobs have finished. Total Time taken: {:.3f} s'.format(end_time-start_time))  
\ No newline at end of file
