@@ -86,14 +86,14 @@ BASE_STYLE = {
 DEFAULT_STYLE = {
     'prefit_1': {
         'linewidth': 2,
-        'color': '#000155',
+        'edgecolor': '#000155',
         'alpha': 1,
         'hatch': None,
         'fill': False,
     },
     'prefit_2':{
         'linewidth': 2,
-        'color': '#027734',
+        'edgecolor': '#027734',
         'alpha': 1,
         'hatch': None,
         'fill': False,
@@ -341,7 +341,7 @@ class NPRankingPlot:
         
     @staticmethod
     def plot_impact(axis, data, show_prefit:bool=True, show_postfit:bool=True, correlation:bool=True, onesided:bool=True, 
-                    relative:bool=False, height:float=1.0, spacing:float=0., **styles):
+                    relative:bool=False, height:float=1.0, spacing:float=0., **styles):     
         n = len(data)  
         impact_types = []
         if show_postfit:
@@ -402,6 +402,7 @@ class NPRankingPlot:
                 # put type1 bar above by default and type2 bar below
                 y_type1[all_neg] = y_type1[all_neg] + w/2
                 y_type1[all_pos] = y_type1[all_pos] + w/2
+
             # negative-valued type 1
             axis.bar(x_neg_type1, height=h[neg_type1], 
                      width=abs(x_neg_type1), bottom=y_type1[neg_type1], 
@@ -417,7 +418,7 @@ class NPRankingPlot:
             # positive-valued type 2
             axis.bar(x[pos_type2], height=h[pos_type2], 
                      width=x_pos_type2, bottom=y_type2[pos_type2], 
-                     align='edge', **style_type2)  
+                     align='edge', **style_type2)
             
     @staticmethod
     def draw_np_labels(axis, data, padding:int=7, height:float=1.0):
@@ -434,7 +435,7 @@ class NPRankingPlot:
             extra_text: extra labels below the ATLAS label, use "//" as newline delimiter
         """
         x_logo = styles['logo']['x']
-        fontsize_logo = styles['logo']['fontsize'] 
+        fontsize_logo = styles['logo']['fontsize']
         _, _, y_curr, _ = draw_ATLAS_label(axis, x=x_logo, y=-height, transform_y='data', fontsize=fontsize_logo)
         dy = styles['label']['dy']        
         x_label = styles['label']['x']
@@ -491,7 +492,7 @@ class NPRankingPlot:
         else:
             ax2.set_xlabel(r"$\Delta$ " + display_poi , **style['xaxis_label'], labelpad=10)
         if sigma_bands:
-            shade, show_prefit, show_postfit = False, False, False            
+            shade, show_prefit, show_postfit = False, False, False
         n = len(data)
         NPRankingPlot.plot_impact(ax2, data, show_prefit=show_prefit, show_postfit=show_postfit,
                                   correlation=correlation, onesided=onesided, relative=relative,
@@ -530,7 +531,7 @@ class NPRankingPlot:
             raise ValueError("no data to process")
         has_impact = any('impact' in i for i in self.data)
         if not has_impact:
-            ranking, threshold, show_prefit, show_postfit = False, False, False, False
+            ranking, threshold, show_prefit, show_postfit = False, False, False, False           
         processed_data = self.get_processed_data(self.data, num=n_rank, threshold=threshold, ranking=ranking)
         num_data = len(processed_data)
         if not num_data:
