@@ -388,7 +388,8 @@ def plot_distributions(data, xlabel='', ylabel='',
     ax.set_ylim(limits[0], limits[1]*1.2)
     ax.set_xlabel(xlabel, fontsize=base_size*1.6,  labelpad=base_size*0.7)
     ax.set_ylabel(ylabel, fontsize=base_size*1.6,  labelpad=base_size*0.7)
-    format_axis_ticks(ax, major_length=14, minor_length=7, width=1.3, labelsize=base_size*1.4)
+    format_axis_ticks(ax, major_length=14, minor_length=7, major_width=1.3, 
+                      minor_width=1.3, labelsize=base_size*1.4)
     if atlas_label:
         draw_ATLAS_label(ax, fontsize=base_size*1.8)
     ax.legend(fontsize=base_size*1.2)
